.\" Copyright (C) 1995, 1996, 1997, 1998
.\" Berwin A. Turlach <bturlach@stats.adelaide.edu.au>
.\" $Id: solve.QP.d,v 1.7 1998/07/23 05:12:20 bturlach Exp $
.\"
.\" This library is free software; you can redistribute it and/or
.\" modify it under the terms of the GNU Library General Public
.\" License as published by the Free Software Foundation; either
.\" version 2 of the License, or (at your option) any later version.
.\" 
.\" This library is distributed in the hope that it will be useful,
.\" but WITHOUT ANY WARRANTY; without even the implied warranty of
.\" MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
.\" Library General Public License for more details.
.\" 
.\" You should have received a copy of the GNU Library General Public
.\" License along with this library; if not, write to the Free Software
.\" Foundation, Inc., 59 Temple Place, Suite 330, Boston,
.\" MA 02111-1307 USA
.\"
.BG
.FN solve.QP
.TL
Solve a Quadratic Programming Problem
.DN
Routine to solve the following Quadratic Programming problem:
        minimize -d^T b + 1/2 b^T D b 
        where    A^T b >= b0
.CS
solve.QP(Dmat, dvec, Amat, bvec, meq=0, factorized=F)
.RA
.AG Dmat
matrix appearing in the quadratic function to be minimized.
.AG dvec
vector appearing in the quadratic function to be minimized.
.AG Amat
matrix defining the constraints under which we want to
minimize the quadratic function.
.OA
.AG bvec
vector holding the values of b0 (defaults to zero).
.AG meq
the first `meq' constraints are treated as equality constraints,
all further as inequality constraints (defaults to 0).
.AG factorized
logical flag: if `TRUE', then we are passing
R^-1 (where D = R^T R) instead of the matrix D  in the argument `Dmat'.
.RT
a list with the following components:
.AG solution
vector containing the solution of the quadratic programming problem.
.AG value
scalar, the value of the quadratic function at the solution
.AG unconstrained.solution
vector containing the unconstrained minimizer of the quadratic function.
.AG iterations
vector of length 2, the first component contains the number of
iterations the algorithm needed, the second indicates how often
constraints became inactive after becoming active first.
.AG iact
vector with the indices of the active constraints at the solution. 
.DT
This routine implements the dual method of Goldfarb and Idnani (1982,
1983) for solving quadratic programming problems.
.SH REFERENCES
Goldfarb, D. and Idnani, A.  (1982).
Dual and Primal-Dual Methods for Solving Strictly Convex
Quadratic Programs.  In 
.ul
Numerical Analysis
J.P. Hennart, ed. Springer-Verlag, Berlin. pp. 226-239.
.PP
Goldfarb, D. and Idnani, A. (1983).
A numerically stable dual method for solving strictly convex quadratic
programs.
.ul
Mathematical Programming
\fB27\fR, 1-33.
.SA
`solve.QP.compact'
.EX
#
# Assume we want to minimize: -(0 5 0) %*% b + 1/2 b^T b
# under the constraints:      A^T b >= b0
# with b0 = (-8,2,0)^T
# and      (-4  2  0) 
#      A = (-3  1 -2)
#          ( 0  0  1)
# we can use solve.QP as follows:
#
Dmat       <- matrix(0,3,3)
diag(Dmat) <- 1
dvec       <- c(0,5,0)
Amat       <- matrix(c(-4,-3,0,2,1,0,0,-2,1),3,3)
bvec       <- c(-8,2,0)
solve.QP(Dmat,dvec,Amat,bvec=bvec)

.KW optimize
.WR
